# -*- coding: utf-8 -*-
"""
Questo modulo implementa una formica in grado di seguire
un percorso più un metodo per generare un percorso
casuale.
"""
import random

def generate_map(x, y) :
  """
  Genera un percorso casuale su una mappa X x Y.
  
    Args:
      X (int) larghezza della mappa
      Y (int) altezza della mappa
    
    Return:
      Una griglia X x Y contenente un percorso per la formica digitale.
  """
  mappa = [[0 for i in range(x)] for j in range(y) ] # Inizializza mappa vuota.
  i , j = 1, 1 # Posizione iniziale.
  mappa[j][i] = 1
  avaiable = ['right'] # Il primo passo va eseguito sempre a destra per semplicità.
  
  while len(avaiable) > 0 :
    next = random.sample(avaiable,1)
    
    if next[0] == 'up' :
      j -= 1
    elif next[0] == 'down' :
      j += 1
    elif next[0] == 'left' :
      i -= 1
    elif next[0] == 'right' :
      i += 1
      
    mappa[j][i] = 1
         
    avaiable = pick_avaiable(mappa,i,j)
  
  else :
    
    mappa[j][i] = 2 # Alla fine setta il punto obiettivo.
  
  return mappa
    
def pick_avaiable(mappa, i, j) :
  """
  Funzione ausiliaria per la generazione della mappa.
  Data una mappa e la posizione attuale restituisce le direzioni
  percorribili in accordo con la regola di non sovrapposizione.
  
    Args:
      mappa (mappa) La mappa corrente
      i (int) attuale posizione x
      j (int) attuale posizione y
      
    Return:
      Una lista di possibili direzioni.
  """
  new_avaiable = []
   
  if j > 1 :
    t = mappa[j-1][i-1] == 0 and mappa[j-1][i] == 0 and mappa[j-1][i+1] == 0
    t = t and mappa[j-2][i-1] == 0 and mappa[j-2][i] == 0 and mappa[j-2][i+1] == 0
    if t:
      new_avaiable.append('up')
  if j < (len(mappa) - 2) :
    t = mappa[j+1][i-1] == 0 and mappa[j+1][i] == 0 and mappa[j+1][i+1] == 0
    t = t and mappa[j+2][i-1] == 0 and mappa[j+2][i] == 0 and mappa[j+2][i+1] == 0
    if t:
      new_avaiable.append('down')
  if i > 1 :
    t = mappa[j-1][i-1] == 0 and mappa[j][i-1] == 0 and mappa[j+1][i-1] == 0
    t = t and mappa[j-1][i-2] == 0 and mappa[j][i-2] == 0 and mappa[j+1][i-2] == 0
    if t:
      new_avaiable.append('left')
  if i < (len(mappa[0]) - 2) :
    t = mappa[j-1][i+1] == 0 and mappa[j][i+1] == 0 and mappa[j+1][i+1] == 0
    t = t and mappa[j-1][i+2] == 0 and mappa[j][i+2] == 0 and mappa[j+1][i+2] == 0
    if t:
      new_avaiable.append('right')
      
  return new_avaiable
  
class Ant(object) :
  """
  Questa classe implementa la formica digitale.
  """
  
  VERSO = {'up': 0, 'right': 1, 'down': 2, 'left':3}
  
  def __init__(self, mappa) :
    self.x = 1
    self.y = 1
    self.mappa = mappa
    self.bit = False
    self.verso = self.VERSO['right']
    
  def r_rotate(self) :
    """
    Ruota la formica a destra.
    """
    self.verso += 1
    if self.verso == 4 :
      self.verso = 0
      
  def l_rotate(self) :
    """
    Ruota la formica a sinistra. Non usato attualmente.
    """
    self.verso -= 1
    if self.verso == -1 :
      self.verso = 3
      
  def forward(self) :
    """
    Fa avanzare di una casela la formica.
    """
    if self.verso == self.VERSO['up'] :
      self.y -= 1
    if self.verso == self.VERSO['down'] :
      self.y += 1
    if self.verso == self.VERSO['left'] :
      self.x -= 1
    if self.verso == self.VERSO['right'] :
      self.x += 1    
    
  def est_ferormone(self) :
    """
    Restituisce 1 se la casella daavanti al formica contiene
    del ferormone, 2 se è la casella obiettivo e 0 se la cella è vuota.
    """
    if self.verso == self.VERSO['up'] :
      return self.mappa[self.y-1][self.x]
    if self.verso == self.VERSO['down'] :
      return self.mappa[self.y+1][self.x]
    if self.verso == self.VERSO['left'] :
      return self.mappa[self.y][self.x -1]
    if self.verso == self.VERSO['right'] :
      return self.mappa[self.y][self.x + 1]
 
  def move(self) :
    """
    Cervello della formica. Elabora il comportamento in base
    alle regole di produzione sotto elencate.
    """
    if self.est_ferormone() == 2 :
      self.forward()
      return 'COMPLETE'
    if self.est_ferormone() and not self.bit :
      self.forward()
      return 'S1'
    if not self.est_ferormone() and not self.bit :
      self.bit = True
      self.r_rotate()
      return 'S2'
    if self.est_ferormone() and self.bit :
      self.bit = False
      self.forward()
      return 'S3'
    if not self.est_ferormone() and self.bit :
      self.r_rotate()
      self.r_rotate()
      self.bit = False
      return 'S4'
      
  def print_map_ant(self) :
    """
    Stampa la mappa del percorso contrassegnando
    la formica con una X.
    """
    print "-"*(len(self.mappa[0])*3)
    for j in range(len(self.mappa)) :
      for i in range(len(self.mappa[j])) :
        if i == self.x and j == self.y :
          print 'X ',
        elif self.mappa[j][i] == 0 :
          print "  ",
        else :
          print "%d " % self.mappa[j][i],
      print ":"
    print "-"*(len(self.mappa[0])*3)